ad_library {

    Videos Library

    @creation-date 2006-09-21
    @author Alessandro Landim <alessandro.landim@gmail.com>
    @cvs-id $Id: videos-procs.tcl,v 1.21 2006/08/08 21:26:52 donb Exp $

}

namespace eval videos {}


ad_proc -public videos::new {
    {-tmp_filename:required}
    {-name:required}
    {-item_id ""}
    {-n_bytes:required}
    {-guessed_file_type:required}
    {-description ""}
    {-queue 0}
    {-user_id:required}
    {-package_id:required}
    {-creation_ip ""}
    {-image_item_id ""}
    {-image_tmp ""}
    {-image_n_bytes ""}
} {
    create a new video
} {

	if {[string equal "" $creation_ip]} {
		set creation_ip [ad_conn peeraddr]
	}
	
	set suggest_name [lang::util::suggest_key $name]
	
	##add video
	
	if {![exists_and_not_null item_id]} {
		set item_id [content::item::new \
						-name "video-$package_id-$user_id-$suggest_name-$n_bytes" \
						-parent_id $package_id \
						-content_type videos_object \
						-package_id $package_id \
						-creation_user $user_id \
						-creation_ip $creation_ip]
	}
	
	
	set revision_id [cr_import_content \
                                 -item_id $item_id \
                                 -storage_type file \
                                 -creation_user $user_id \
                                 -creation_ip $creation_ip \
				 -description "video id: $item_id" \
				 -package_id $package_id \
                                 $package_id \
                                 $tmp_filename \
                                 $n_bytes \
                                 $guessed_file_type \
                                 video-$package_id-$user_id-$suggest_name-$n_bytes]
 	
       	item::publish -item_id $item_id -revision_id $revision_id
	
	##add image 

	set image_guessed_file_type [ns_guesstype $image_tmp] 
        ns_log notice "image: $image_guessed_file_type"	
	if {![string equal $image_item_id ""]} {


		set image_revision_id [cr_import_content \
                                 	-item_id $image_item_id \
					-image_only \
                                 	-storage_type file \
                                 	-creation_user $user_id \
                                 	-creation_ip $creation_ip \
				 	-description "image of video id: $item_id" \
				 	-package_id $package_id \
                                 	$package_id \
                                 	$image_tmp \
                                 	$image_n_bytes \
                                 	$image_guessed_file_type \
                                 	video-image-$item_id]
       
		item::publish -item_id $image_item_id -revision_id $image_revision_id

	        set user_portrait_rel_id [relation::get_id -object_id_one $item_id -object_id_two $image_item_id -rel_type "videos_image_thumbnail_rel"]
		 
		if {[empty_string_p $user_portrait_rel_id]} {
        		db_exec_plsql create_rel_image {}
   		 }

	}

	if {[string equal $queue 1]} {
		db_exec_plsql video_insert ""
		videos::video_queue -item_id $item_id
		permission::grant -party_id $user_id -object_id $item_id -privilege admin
	}

	return $item_id

}


ad_proc -public videos::convert {
    {-item_id:required}
} {
    convert video to flv
} {
	set path [cr_fs_path CR_FILES]
	set revision_id [db_string get_live_revision ""]
	set filename [db_string write_file_content ""]

	set video_tmp_filename "/tmp/videos-[ns_rand 1000000].flv"
	set video_tmp_filename1 "/tmp/videos-[ns_rand 1000000].wmv"
	set image_tmp_filename "/tmp/image-videos-[ns_rand 1000000].jpeg"
	set image_tmp_filename_film "/tmp/image-videos-2[ns_rand 1000000].jpeg"
	
	set guessed_file_type_orig  [ns_guesstype $filename]

	if {![catch {exec ffmpeg -sameq -i $filename -acodec ac3 -crf 10 $video_tmp_filename1} errorMsg]} {
			# do nothing
			ns_log notice "error coverting video to wmv "
	} else {

		if {![catch {exec mencoder $video_tmp_filename1 -ofps 25 -o $video_tmp_filename -of lavf -oac mp3lame -lameopts abr:br=64 -srate 22050 -ovc lavc -lavfopts i_certify_that_my_video_stream_does_not_use_b_frames -lavcopts vcodec=flv:keyint=50:vbitrate=300:mbd=2:mv0:trell:v4mv:cbp:last_pred=3 -vop scale=480:360} errorMsg]} {
			# do nothing
			ns_log notice "error coverting video to flv "
		} else { 
			
			if {![catch {set convert_image [exec ffmpeg -an -y -i $video_tmp_filename -f mjpeg -s 90x70 -ss 5 -vframes 1 $image_tmp_filename]} errorMsg]} {
			# do nothing
			ns_log notice "error coverting image thumbnail  "

		} else {
				exec convert -composite $image_tmp_filename "[acs_root_dir]/packages/videos/www/resources/film.png" $image_tmp_filename_film 
				
				set image_tmp_size [file size $image_tmp_filename_film]
		
				set n_bytes [file size $video_tmp_filename]
				set guessed_file_type [ns_guesstype $video_tmp_filename] 

				exec flvtool2 -U $video_tmp_filename	

				videos::get -item_id $item_id -array orig_video
		
			        set image_item_id [videos::get_image_id \
								-item_id $item_id \
								-package_id $orig_video(package_id) \
								-creation_user $orig_video(creation_user) \
								-creation_ip $orig_video(creation_ip)]

				videos::new -tmp_filename $video_tmp_filename \
			    		-item_id $item_id -n_bytes $n_bytes \
			    		-guessed_file_type $guessed_file_type \
			    		-description $orig_video(video_description) \
			    		-name $orig_video(video_name) \
			    		-package_id $orig_video(package_id) \
			    		-user_id $orig_video(creation_user) \
			    		-creation_ip $orig_video(creation_ip) \
			    		-image_item_id $image_item_id \
			    		-image_tmp $image_tmp_filename_film \
			    		-image_n_bytes $image_tmp_size
			}
		}

	}

	file delete $video_tmp_filename
	file delete $image_tmp_filename_film
	file delete $video_tmp_filename1
	file delete $image_tmp_filename

	videos::delete_video_queue -item_id $item_id	
}

ad_proc -public videos::video_queue {
    -item_id:required
} {
    This proc add video into queue to convert. 
} {
 	db_dml insert_video_queue {}

}

ad_proc -public videos::delete_video_queue {
    -item_id:required
} {
    This proc add video into queue to convert. 
} {
 	db_dml delete_video_queue {}

}

ad_proc -public videos::get {
    -item_id:required
    -array:required
} {
    This proc retrieves a video. 
} {
    upvar 1 $array row
    db_1row videos_select {} -column_array row
}

ad_proc -public videos::convert_queue {} {
    This proc retrieves a video. 
} {
    
    set items_id [db_list select_queue {select item_id from videos_queue}]
    foreach item_id $items_id {
	videos::convert -item_id $item_id
    }

}

ad_proc -public videos::get_image_id {
    -item_id:required
    -package_id:required
    -creation_user:required
    -creation_ip:required
} {
    This proc retrieves a video. 
} {
 
	set image_item_id [db_string select_item_image "" -default ""]

	if {[string equal "" $image_item_id]} {
		set image_item_id [content::item::new \
						-name "video-image-$item_id" \
						-parent_id $package_id \
						-content_type image \
						-package_id $package_id \
						-creation_user $creation_user \
						-creation_ip $creation_ip]
	}
	return $image_item_id
}
